"""Script for running a tsunami inundation scenario for Sydney, NSW, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputdir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a simulated submarine landslide.

Ole Nielsen and Duncan Gray, GA - 2005 and Adrian Hitchman and Jane Sexton, GA - 2006
"""


#-------------------------------------------------------------------------------
# Import necessary modules
#-------------------------------------------------------------------------------

# Standard modules
import os
import time

# Related major packages
from pyvolution.shallow_water import Domain, Reflective_boundary
from pyvolution.data_manager import convert_dem_from_ascii2netcdf, dem2pts
from pyvolution.combine_pts import combine_rectangular_points_files 
from pyvolution.pmesh2domain import pmesh_to_domain_instance
from pmesh.mesh_interface import create_mesh_from_regions

# Application specific imports
import project                           # Definition of file names and polygons
from pyvolution.smf import slump_tsunami # Function for submarine mudslide 



#-------------------------------------------------------------------------------   
# Prepare bathymetric and topographic data
#-------------------------------------------------------------------------------

# filenames
coarsedemname = project.coarsedemname + '.pts'
finedemname = project.finedemname + '.pts'
combineddemname = project.combineddemname + '.pts'
meshname = project.meshname+'.msh'


# combining the coarse and fine data
combine_rectangular_points_files(finedemname,
                                 coarsedemname,
                                 combineddemname)



#-------------------------------------------------------------------------------   
# Setup computational domain
#-------------------------------------------------------------------------------

# Interior regions and mesh resolutions
interior_res = 5000
interior_regions = [[project.harbour_polygon_2, interior_res],
                    [project.botanybay_polygon_2, interior_res]]


create_mesh_from_regions(project.diffpolygonall,
                         boundary_tags= {'bottom': [0], 
                                         'right1': [1],
                                         'right0': [2],
                                         'right2': [3],
                                         'top': [4],
                                         'left1': [5],
                                         'left2': [6],
                                         'left3': [7]},
                         maximum_triangle_area=100000,
                         filename=meshname,           
                         interior_regions=interior_regions)



domain = pmesh_to_domain_instance(meshname, Domain,
                                  use_cache = True,
                                  verbose = True)

print 'Number of triangles = ', len(domain)
print 'The extent is ', domain.get_extent()

domain.set_name(project.basename)
domain.set_datadir(project.outputdir)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])


#-------------------------------------------------------------------------------
# Set up scenario (tsunami_source is a callable object used with set_quantity)
#-------------------------------------------------------------------------------

tsunami_source = slump_tsunami(length=30000.0,
                               depth=400.0,
                               slope=6.0,
                               thickness=176.0, 
                               radius=3330,
                               dphi=0.23,
                               x0=project.slump_origin[0], 
                               y0=project.slump_origin[1], 
                               alpha=0.0, 
                               domain=domain)


#-------------------------------------------------------------------------------
# Setup initial conditions
#-------------------------------------------------------------------------------

domain.set_quantity('stage', tsunami_source)
domain.set_quantity('friction', 0.0) 
domain.set_quantity('elevation',
                    filename = combineddemname,
                    use_cache = True,
                    verbose = True)


#-------------------------------------------------------------------------------
# Setup boundary conditions (all reflective)
#-------------------------------------------------------------------------------

print 'Available boundary tags', domain.get_boundary_tags()

Br = Reflective_boundary(domain)
domain.set_boundary( {'bottom': Br, 'right1': Br, 'right0': Br,
                      'right2': Br, 'top': Br, 'left1': Br,
                      'left2': Br, 'left3': Br} )


#-------------------------------------------------------------------------------
# Evolve system through time
#-------------------------------------------------------------------------------

import time
t0 = time.time()

for t in domain.evolve(yieldstep = 120, duration = 18000):
    print domain.timestepping_statistics()
    print domain.boundary_statistics(tags = 'bottom')    
    
print 'That took %.2f seconds' %(time.time()-t0)
